{

Client dataset creator 1.0 by Andrea Raimondi
Copyright (C) 2002 by Andrea Raimondi

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
or go to http://www.gnu.org/licenses/gpl.html

You can reach me via EMail at: rainaple@tin.it or mrpc@supereva.it
Or via normal mail at the following address:

Andrea Raimondi
Via Monte di Dio 66
80132 Napoli
ITALY

}
unit MainCode;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, DBGrids, Mask, DB, DBClient, ComCtrls;

type
  TFieldDefRec = record
    FName : String;
    FType : TFieldType;
    FSize : Integer;
  end;

  TMainFrm = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    DBGrid1: TDBGrid;
    CreateBtn: TButton;
    CloseBtn: TButton;
    ClientDataSet1: TClientDataSet;
    SaveToFileBtn: TButton;
    SchemaFileEdit: TEdit;
    BreowseBtn: TButton;
    OpenDialog1: TOpenDialog;
    DataSource1: TDataSource;
    OperationStatus: TStatusBar;
    procedure CloseBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BreowseBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CreateBtnClick(Sender: TObject);
    procedure SaveToFileBtnClick(Sender: TObject);
  private
    { Private declarations }
    FLines : TStringList;
    OldCursor : TCursor;
    procedure LoadSchemaFile;
    procedure DecodeLine( FieldLine : String;var FieldDefRec : TFieldDefRec );
    procedure AddFieldToClientDataset( FR : TFieldDefRec );
  public
    { Public declarations }
    procedure SaveCursor;
    procedure CreateDataset;
    procedure RestoreCursor;
    procedure FlushFields;
  end;

var
  MainFrm: TMainFrm;

implementation

uses FieldConv,CDSSaveAsFileCode, CDSStringList;

{$R *.dfm}

procedure TMainFrm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TMainFrm.FormCreate(Sender: TObject);
begin
  FLines := TCommentedStringList.Create;
  TCommentedStringList( FLines ).CommentString := '//';
end;

procedure TMainFrm.BreowseBtnClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    FlushFields;
    FLines.Clear;
    SchemaFileEdit.Text := OpenDialog1.FileName;
    SaveCursor;
    Screen.Cursor := crHourGlass;
    LoadSchemaFile;
    RestoreCursor;
  end;
end;

procedure TMainFrm.LoadSchemaFile;
begin
  FLines.LoadFromFile( SchemaFileEdit.Text );
  OperationStatus.SimpleText := 'Loaded '+SchemaFileEdit.Text;
end;

procedure TMainFrm.DecodeLine(FieldLine: String;
                              var FieldDefRec: TFieldDefRec);

var FieldConverter : TFieldConversion;
    TheStrings : array[ 0..2 ] of String;

    procedure ManualDecode;
      var StrPos : Integer;
          Str_Index : Integer;
    begin
      for Str_Index := 0 to 2 do
      begin
        StrPos := Pos( ',',FieldLine );
        if StrPos > 0 then
        begin
          TheStrings[ Str_Index ] := Copy( FieldLine,1,Pred( StrPos ) );
          Delete( FieldLine,1,StrPos );
        end
        else TheStrings[ Str_Index ]:= FieldLine;
      end;
    end;

begin
  with FieldDefRec do
  begin
    ManualDecode;
    FName := TheStrings[ 0 ];
    FieldConverter := TExtendedFieldConversion.Create;
    FieldConverter.FieldTypeStr := TheStrings[ 1 ];
    FType := FieldConverter.FieldType;
    FSize := StrToInt( TheStrings[ 2 ] );
    FieldConverter.Free;
  end;
end;

procedure TMainFrm.FormDestroy(Sender: TObject);
begin
  FLines.Free;
end;

procedure TMainFrm.AddFieldToClientDataset(FR: TFieldDefRec);
begin
  with ClientDataset1.FieldDefs.AddFieldDef do
  begin
    Name := FR.FName;
    DataType := FR.FType;
    Size := FR.FSize;
  end;
end;

procedure TMainFrm.CreateDataset;
var FR : TFieldDefRec;
    Line_Index : Integer;
begin
  for Line_Index := 0 to Pred( FLines.Count ) do
  begin
    SaveCursor;
    Screen.Cursor := crHourGlass;
    DecodeLine( FLines[ Line_Index ],FR );
    OperationStatus.SimpleText := 'Adding '+FR.FName+' field';
    try
      AddFieldToClientDataset( FR );
    except
      on Exception do ShowMessage( 'Exception on field '+FR.FName );
    end;
    Sleep( 500 );
    RestoreCursor;
  end;
  SaveCursor;
  Screen.Cursor := crHourGlass;
  ClientDataset1.CreateDataSet;
  OperationStatus.SimpleText := 'Dataset created';
  RestoreCursor;
end;

procedure TMainFrm.CreateBtnClick(Sender: TObject);
begin
  CreateDataset;
end;

procedure TMainFrm.RestoreCursor;
begin
  Screen.Cursor := OldCursor;
end;

procedure TMainFrm.SaveCursor;
begin
  OldCursor := Screen.Cursor;
end;

procedure TMainFrm.SaveToFileBtnClick(Sender: TObject);
begin
  with TCDSSaveAsFileFrm.Create( Self ) do
  begin
    ShowModal;
    Free;
  end;
end;

procedure TMainFrm.FlushFields;
var Field_Index : Integer;
begin
  for Field_Index := 0 to Pred( ClientDataset1.FieldCount ) do
  begin
    // The clientdataset numbers its fields dinamically, so
    // to be sure not to go out of bounds, I erase the
    // first element only.
    ClientDataset1.FieldDefs.Delete( 0 );
  end;
  ClientDataset1.Active := False;
end;

end.
